// SimpleParticleSystem original by Daniel Schiffman (see: http://processing.org/learning/topics/simpleparticlesystem.html )
// Modified by Ingmar Drewing

Fields fields;
ParticleSystem ps;
int currentY = 0;

void setup() {
  size(400, 400);
  background(255);
  colorMode(RGB, 255, 255, 255, 100);
  ps = new ParticleSystem(1, new PVector(0,0,0));
  fields = new Fields();
  smooth();
}

void draw() {
  ps.run();
  ps.addParticle(random(0,width), random(0,height));
  currentY = currentY >= height ? 0 : currentY + 2 ;
}

// A class to describe a group of Particles
// An ArrayList is used to manage the list of Particles 

class ParticleSystem {

  ArrayList particles;    // An arraylist for all the particles
  PVector origin;        // An origin point for where particles are born

  ParticleSystem(int num, PVector v) {
    particles = new ArrayList();              // Initialize the arraylist
    origin = v.get();                        // Store the origin point
    for (int i = 0; i < num; i++) {
      particles.add(new Particle(origin));    // Add "num" amount of particles to the arraylist
    }
  }

  void run() {
    // Cycle through the ArrayList backwards b/c we are deleting
    for (int i = particles.size()-1; i >= 0; i--) {
      Particle p = (Particle) particles.get(i);
      p.run();
      if (p.dead()) {
        particles.remove(i);
      }
    }
  }

  void addParticle() {
    particles.add(new Particle(origin));
  }
  
  void addParticle(float x, float y) {
    particles.add(new Particle(new PVector(x,y)));
  }

  void addParticle(Particle p) {
    particles.add(p);
  }

  // A method to test if the particle system still has particles
  boolean dead() {
    if (particles.isEmpty()) {
      return true;
    } else {
      return false;
    }
  }
}


// A simple Particle class

class Particle {
  PVector loc;
  PVector lastLoc;
  PVector vel;
  PVector acc;
  float r;
  float timer;
  
  Particle(PVector l) {
    acc = new PVector(0,0.05,0);
    vel = new PVector(random(-1,1),random(-2,0),0);
    loc = l.get();
    timer = 100.0;
  }

  void run() {
    update();
    render();
  }

  // Method to update location
  void update() {
    fields.updateParticle( this );
    timer -= 1.0;
  }

  // Method to display
  void render() {
    if( lastLoc != null ){
      int c = round(map( timer, 0, 100, 0, 255 ));
      stroke( c ); 
      beginShape();
      vertex( lastLoc.x, lastLoc.y );
      vertex( loc.x, loc.y );
      endShape();
    }
    lastLoc = new PVector( loc.x, loc.y );
  }
  
  // Is the particle still useful?
  boolean dead() {
    if (timer <= 0.0) {
      return true;
    } else {
      return false;
    }
  }
}


// Creating a Field to influence the particles behaviour

class Fields {
  ArrayList fields;
  
  Fields(){
   fields = new ArrayList();
   addField(); 
  }
  
  void addField(){
    fields.add( new Attractor( 50 ) );
  }
  
  void updateParticle( Particle p ){
    p.loc.y += random(-1,1);
    p.loc.x += random(-1,1);
    for( int i=0; i < fields.size(); i++){
      Attractor a = (Attractor) fields.get(i);
      a.influence( p );
    }
  }
}

class Attractor {
  float x = width / 2;
  float y = height / 2;
  float r = 20;
  
  Attractor(float r_param){
    r = r_param;
  } 
  
  void influence( Particle p ){
    PVector d = new PVector( x - p.loc.x, y - p.loc.y );
    float distance = d.mag();
    d.normalize();
    d.mult( 1/sqrt(2 * PI) * exp( -((distance/200) * (distance/200))/ 2) );
    p.loc.add( d );
  }
}
